package TamagotchiPackage;

import java.util.*;
import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

import TamagotchiOntology.*;

/**
 * The behaviours of an agent of the Tamagotchi type
 */

class Message_handler extends CyclicBehaviour {
/**
 * Is waiting for incoming messages and forwards the message
 * to an appropriate behaviour
 */

 /** Placeholder for the received message */
 private ACLMessage receivedMessage;

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** Constructor of the behaviour */
 public Message_handler(Tamagotchi agent) {
   super(agent);
   thisTamagotchi = agent;
 }

 /** Actual implementation of the behaviour */
 public void action() {
  /** Receive the message */
  ACLMessage msg = myAgent.receive();
  if (msg == null) {
    block();
  } else {
    if (msg.getPerformative() == ACLMessage.INFORM &&
        msg.getSender().equals(thisTamagotchi.getFriend()) &&
        msg.getContent().equals("Please come")) {
        Behaviour b = new Visit_another_Tamagotchi(thisTamagotchi);
        myAgent.addBehaviour(b);
      }
    }
  }
}

class Mood_checker extends CyclicBehaviour {
/**
 * Is continuously checking beliefs of a Tamagotchi and invokes
 * an appropriate behaviour if a relevant belief has changed
 */

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** Constructor of the behaviour */
 public Mood_checker(Tamagotchi agent) {
   super(agent);
   thisTamagotchi = agent;
 }

 /** Actual implementation of the behaviour */
 public void action() {
   if (thisTamagotchi.getHappinessLevel() < 2 &&
       !thisTamagotchi.ownerInformed()) {
     thisTamagotchi.getUserInterface().appendMessage("I am unhappy");
     thisTamagotchi.ownerIsInformed();
   }
 }
}

class Connect_with_another_Tamagotchi extends OneShotBehaviour {
/**
 * Establishes a connection between the two Tamagotchis
 * with co-located Tamagotchi Shells (environments)
 */

 /** MyTamagotchiAgent */
 Tamagotchi thisTamagotchi;

 /** The message to be sent */
 private ACLMessage msg;

 /** Constructor of the behaviour */
 public Connect_with_another_Tamagotchi(Tamagotchi tamagotchi) {
   super(tamagotchi);
   thisTamagotchi = tamagotchi;
 }

 /** Actual implementation of the behaviour */
 public void action() {
   TamagotchiShell shell = thisTamagotchi.getTamagotchiShell();
   thisTamagotchi.setFriend(shell.Communicator.connect());
   /** Prepare and send the INFORM message */
   msg = new ACLMessage(ACLMessage.INFORM);
   msg.setSender(new AID(myAgent.getLocalName(), AID.ISLOCALNAME));
   msg.addReceiver(thisTamagotchi.getFriend());
   msg.setContent("Could I come to your place?");
   myAgent.send(msg);
   thisTamagotchi.getUserInterface().appendMessage
       ("Connecting with another Tamagotchi");
 }
}

class Visit_another_Tamagotchi extends SequentialBehaviour {
/**
 * Implements a visit between the two interconnected Tamagotchis
 * where the initiating Tamagotchi goes to the hosting
 * Tamagotchi's environment
 */

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** The message to be sent */
 private ACLMessage msg;

 /** Constructor of the behaviour */
 public Visit_another_Tamagotchi(Tamagotchi tamagotchi) {
   super(tamagotchi);
   thisTamagotchi = tamagotchi;
 }

 /** Actual implementation of the behaviour */
 public void onStart() {
   /** Retrieve the local environment */
   TamagotchiShell shell = thisTamagotchi.getTamagotchiShell();
   /** Invoke the visit() operation of the Communicator
    * service */
   if (shell.Communicator.visit()) {
      /** If going to the other Tamagotchi's environment
       * was successful */
      /** Prepare and send the INFORM message */
      thisTamagotchi.getUserInterface().appendMessage("Away");
      msg = new ACLMessage(ACLMessage.INFORM);
      msg.setSender
          (new AID(myAgent.getLocalName(), AID.ISLOCALNAME));
      msg.addReceiver(thisTamagotchi.getFriend());
      msg.setContent("Good day!");
      myAgent.send(msg);
      while (thisTamagotchi.getHappinessLevel() < 2.0) {
        Give_a_present_and_play b = new Give_a_present_and_play
            (thisTamagotchi);
        addSubBehaviour(b);
        b.onStart();
        removeSubBehaviour(b);
      }
    }
    /** Retrieve the local environment */
    TamagotchiShell shell = thisTamagotchi.getTamagotchiShell();
    if (shell.Communicator.returnHome()) {
     thisTamagotchi.getUserInterface().appendMessage("Back");
    }
  }
}

class Give_a_present_and_play extends SequentialBehaviour {
/**
 * The visiting Tamagotchi alternately gives a present and plays
 * with the hosting Tamagotchi until the happiness level of the
 * first Tamagotchi has improved
 */

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** Constructor of the behaviour */
 public Give_a_present_and_play (Tamagotchi tamagotchi) {
   super(tamagotchi);
   thisTamagotchi = tamagotchi;
 }

 /** Actual implementation of the behaviour */
 public void onStart () {
   Give_a_present b1 = new Give_a_present(thisTamagotchi);
   addSubBehaviour(b1);
   b1.action();
   removeSubBehaviour(b1);
   Play b2 = new Play(thisTamagotchi);
   addSubBehaviour(b2);
   b2.action();
   removeSubBehaviour(b2);
 }
}

class Give_a_present extends OneShotBehaviour {
/**
 * Gives a present from the visiting Tamagotchi to
 * the hosting one
 */

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** Constructor of the behaviour */
 public Give_a_present (Tamagotchi tamagotchi) {
   super(tamagotchi);
   thisTamagotchi = tamagotchi;
 }

 /** Actual implementation of the behaviour */
 public void action () {
   Present present = thisTamagotchi.getPresent();
   if (present != null) {
       /** Prepare and send the INFORM message */
       ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
       msg.setSender
           (new AID(myAgent.getLocalName(), AID.ISLOCALNAME));
       msg.addReceiver(thisTamagotchi.getFriend());
       try {
         msg.setContentObject(present);
       }
       catch (java.io.IOException ioe) {
         ioe.printStackTrace();
       }
       myAgent.send(msg);
       thisTamagotchi.getUserInterface().appendMessage
           ("Giving a present " + present.getPresentType());
       thisTamagotchi.setHappinessLevel
           (thisTamagotchi.getHappinessLevel() + 0.4);
     }
   }
 }

class Play extends OneShotBehaviour {
/**
 * Simulates playing between the two interconnected Tamagotchis
 * after the visiting Tamagotchi has gone to the hosting
 * Tamagotchi's environment
 */

 /** The "host" Tamagotchi of this behaviour */
 Tamagotchi thisTamagotchi;

 /** Constructor of the behaviour */
 public Play (Tamagotchi tamagotchi) {
   super(tamagotchi);
   thisTamagotchi = tamagotchi;
 }

 /** Actual implementation of the behaviour */
 public void action () {
   /** Prepare and send the INFORM message */
   ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
   msg.setSender
       (new AID(myAgent.getLocalName(), AID.ISLOCALNAME));
   msg.addReceiver(thisTamagotchi.getFriend());
   msg.setContent("Playing");
   myAgent.send(msg);
   thisTamagotchi.getUserInterface().appendMessage("Playing");
   thisTamagotchi.setHappinessLevel
       (thisTamagotchi.getHappinessLevel() + 0.2);
 }
}
